# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess
import unittest

import mock

from imagebuilder import cloud_image


class TestCloudImage(unittest.TestCase):
    def setUp(self):
        super(TestCloudImage, self).setUp()

    def test_parse_ppas_single(self):
        chroot = '/tmp/foo'
        ppalist = ['ppa:foo/bar']
        series = 'precise'
        expect_aptline = [
            'deb http://ppa.launchpad.net/foo/bar/ubuntu precise main',
            'deb-src http://ppa.launchpad.net/foo/bar/ubuntu precise main'
        ]
        aptline = cloud_image._parse_ppas(chroot, ppalist, series)
        self.assertEqual(expect_aptline, aptline)

    def test_parse_ppas_multiple(self):
        chroot = '/tmp/foo'
        ppalist = ['ppa:foo/bar', 'ppa:moo/oink']
        series = 'precise'
        expect_aptline = [
            'deb http://ppa.launchpad.net/foo/bar/ubuntu precise main',
            'deb-src http://ppa.launchpad.net/foo/bar/ubuntu precise main',
            'deb http://ppa.launchpad.net/moo/oink/ubuntu precise main',
            'deb-src http://ppa.launchpad.net/moo/oink/ubuntu precise main'
        ]
        aptline = cloud_image._parse_ppas(chroot, ppalist, series)
        self.assertEqual(expect_aptline, aptline)

    @mock.patch('imagebuilder.cloud_image._chrooted')
    def test_apt_get_update_fails(self, chrooted):
        chrooted.side_effect = subprocess.CalledProcessError(1, 'cmd', 'out')
        with self.assertRaises(subprocess.CalledProcessError):
            cloud_image._apt_get_update('/blah')

    @mock.patch('imagebuilder.cloud_image._chrooted')
    def test_apt_get_update_retries(self, chrooted):
        '''ensure it can apt-get update works after retrying'''
        chrooted.side_effect = [
            subprocess.CalledProcessError(1, 'cmd', 'out'),
            subprocess.CalledProcessError(1, 'cmd', 'out'),
            None,
        ]
        cloud_image._apt_get_update('/blah')
        self.assertEqual(3, chrooted.call_count)
